MAX_PATH = 260

// Error Codes
ERC_HARDWARE            = 2
ERC_OUT_OF_RANGE        = 5
ERC_DISK_CHANGED        = 6
ERC_NOT_FLUSHED         = 7
ERC_DISK_IO             = 8
ERC_INVALID_DISK_NUMBER = 10h
ERC_NOT_FOUND_IN_CACHE  = 12h  // internal
ERC_UNKNOWN_GEOMETRY    = 15h
ERC_DISK_FULL           = 16h

ERC_INVALID_CHANNEL_NUMBER = 18h
ERC_INVALID_MEMORY_ADDRESS = 19h
ERC_INVALID_COUNT          = 1Ah

ERC_NOT_ENOUGH_MEMORY = 21h
ERC_ALIGN             = 24h
ERC_ALREADY_FREE      = 25h
ERC_OVERFLOW          = 26h
ERC_UNDERFLOW         = 27h

ERC_MSG_QUEUE_EMPTY    = 30h
ERC_MSG_QUEUE_FULL     = 31h
ERC_MSG_INVALID_TARGET = 32h

ERC_TOO_MANY_WINDOWS = 33h
ERC_INVALID_WINDOW   = 34h
ERC_CANT_MOVESIZE    = 35h
ERC_UNKNOWN_CLASS    = 36h

ERC_TOO_MANY_PROCS   = 40h
ERC_TOO_MANY_THREADS = 41h

ERC_TIMER_FULL      = 42
ERC_TIMER_NOT_FOUND = 43

ERC_PATH_NOT_FOUND = 44h
ERC_FILE_NOT_FOUND = 45h
ERC_TOO_MANY_OPEN_FILES = 46h
ERC_DIR_ALREADY_EXISTS = 47h
ERC_UNKNOWN_FILESYSTEM = 48h
ERC_DIR_NOT_EMPTY = 49h
ERC_CANT_REMOVE = 4Ah
ERC_DUPLICATE_NAME = 4Bh
ERC_INVALID_FHANDLE = 4Ch
ERC_INVALID_THREAD = 4Dh
ERC_FS = 4Eh
ERC_READ_ONLY = 4Fh
ERC_IN_USE = 50h
ERC_OUT_OF_DISK = 51h
ERC_TOO_MANY_SEARCHES = 52h
ERC_INVALID_SHANDLE = 53h
ERC_CANT_RENAME = 54h

ERC_INVALID_PID = 55h
ERC_STACK_OVERFLOW = 56h
ERC_PE_INVALID_HEADER = 57h
ERC_INVALID_SERVICE = 58h

ERC_TOO_LONG = 59h
ERC_ONLY_STARTER = 5Ah
ERC_INVALID_MEMORY_LOCATION = 5Bh
ERC_INVALID_THREADID = 5Ch
ERC_ACCESS_DENIED = 5Dh
ERC_INVALID_PRIORITY = 5Eh
ERC_INVALID_ATRBS = 5Fh
ERC_INVALID_VALUE = 60h
ERC_INVALID_POINTER = 61h
ERC_INVALID_CHAR = 62h
ERC_DISK_REMOVED = 63h
ERC_TOO_BIG_HOLE = 64h  // internal
ERC_UTF8 = 65h
ERC_BUSY = 66h
ERC_DIR_TOO_BIG = 67h
ERC_LINK = 68h  // internal
ERC_NOT_A_LINK = 69h
ERC_INFINITE_LINK = 70h
ERC_CANT_MOVE = 71h
ERC_FILE_TOO_BIG = 72h
ERC_NOT_FIT_IN_MFT = 73h
ERC_INVALID_LINK = 74h

ERC_PE_IMPORT_DEPTH_OVERFLOW = 75h
ERC_PE_INVALID_ADR = 76h
ERC_PE_NOT_FOUND = 77h
ERC_PE_ORDINAL = 78h

ERC_INVALID_IMAGE = 79h

ERC_NOT_IMPLEMENTED     = 0FFh

// Window = 32 B (winid)
struct WinRec{
  int2 x, y, w, h
  uns4 flags, cr3, hwnd, pbitmap, pid
  uns1 kbd, parent
  uns1 ncheight, desktop
}

// Window Ex (hwnd)
struct WindowExRec{
  uns4 next, prev, first_child, pmenu, ptext, pclass, menusx
  int2 x, y, w, h
  uns4 winid, pid, hinstance
  uns4 exstyle, style, id, parent, wndproc
}

struct WndClassRec{
  int4 cbSize
  uns4 style
  ptr4 lpfnWndProc
  int4 cbClsExtra
  int4 cbWndExtra
  uns4 hInstance
  ptr4 hIcon
  ptr4 hCursor
  uns4 hbrBackground
  ptr4 lpszMenuName
  ptr4 lpszClassName
  ptr4 hIconSm
  ptr4 next // only tryskos
}

struct PointRec{
  int4 x, y
}

struct MessageRec{
  ptr4 hwnd
  uns4 message
  uns4 wParam
  uns4 lParam
  uns4 time
  PointRec pt
}

struct DCRec{
  int2 x0, y0, x1, y1, x2, y2, x, y
  ptr4 pbitmap, pen, brush, rgn, font, lppaintstruct
  uns4 textcolor
}

struct RectRec{
  int4 left, top, right, bottom
}

GDIOBJ_PEN = 1
GDIOBJ_BRUSH = 2
GDIOBJ_RGN = 3
GDIOBJ_FONT = 4

struct PenRec{
  uns4 type, color, size
}

struct BrushRec{
  uns4 type, flags, color1, color2, pbrushfn
  int2 x1, y1, x2, y2
}

struct RgnRec{
  uns4 type, flags
  int2 left, top, right, bottom
}

struct FontRec{
  uns4 type, rotation, pfont
  bool bold, italics, underline, strike
  char name[64]
}

struct PaintStruct {
	ptr4 hdc; // 0
	uns4 fErase; // 4
	RectRec rcPaint; // 8
	uns4 fRestore; // 24
	uns4 fIncUpdate; // 28
	uns1 rgbReserved[32]; // 32
}

// BitBlt
BLACKNESS = 0x42
NOTSRCERASE = 0x1100A6
NOTSRCCOPY = 0x330008
SRCERASE = 0x440328
DSTINVERT = 0x550009
PATINVERT = 0x5A0049
SRCINVERT = 0x660046
SRCAND = 0x8800C6
MERGEPAINT = 0xBB0226
MERGECOPY = 0xC000CA
SRCCOPY = 0xCC0020
SRCMERGE = 0xDD0022
SRCPAINT = 0xEE0086
WHITENESS = 0xFF0062

// GetStockObject
BLACK_BRUSH = 4
DKGRAY_BRUSH = 3
GRAY_BRUSH = 2
HOLLOW_BRUSH = 5
LTGRAY_BRUSH = 1
NULL_BRUSH = 5
WHITE_BRUSH = 0
BLACK_PEN = 7
NULL_PEN = 8
WHITE_PEN = 6
ANSI_FIXED_FONT = 11
ANSI_VAR_FONT = 12
DEVICE_DEFAULT_FONT = 14
DEFAULT_GUI_FONT = 17
OEM_FIXED_FONT = 10
SYSTEM_FONT = 13
SYSTEM_FIXED_FONT = 16
DEFAULT_PALETTE = 15

struct DateTime8{
  uns8 datetime
}

struct SearchRec{
  uns4 FileAttributes
  DateTime8 CreationTime
  DateTime8 LastAccessTime
  DateTime8 LastWriteTime
  uns8 FileSize
  uns8 Reserved
  char FileName[MAX_PATH]
  char AlternateFileName[14]
}

struct SystemTime{
  uns2 wYear
  uns2 wMonth
  uns2 wDayOfWeek
  uns2 wDay
  uns2 wHour
  uns2 wMinute
  uns2 wSecond
  uns2 wMilliseconds
}

MAX_WIN = 255
WIN_NONE = 255
WIN_MAINPANELH = 0
WIN_HEADER = 16
WIN_BORDER = 6
WIN_CORNER = 16
WIN_MINW = 64 + (WIN_BORDER * 2)
WIN_MINH = WIN_HEADER + (WIN_BORDER * 2)
WIN_MENUH = 16

WIN_DEFAULT_X_STEP = 32
WIN_DEFAULT_Y_STEP = 32
WIN_DEFAULT_X_MAX = 250
WIN_DEFAULT_Y_MAX = 250
WIN_BORDERCOLOR = 0xFF0080FF
WIN_HEADERCOLOR = 0xFF0080FF
WIN_FRAMECOLOR = 0xFF000000
WIN_ACTIVEFRAMECOLOR = 0xFFFFFF00
WIN_CLOSECOLOR = 0xFFE04040
WIN_MAXIMIZECOLOR = 0xFFA0A0A0
WIN_LIGHT = 0xFFFFFFFF
WIN_DARK = 0xFF606060

WINFLAGS_VALID = 1
WINFLAGS_FULLSCREEN = 2
WINFLAGS_ALWAYSONTOP = 8
WINFLAGS_CLOSEPRESSED    = 0x10
WINFLAGS_MAXIMIZEPRESSED = 0x20
WINFLAGS_MINIMIZEPRESSED = 0x40

// wp of mousemove
MK_LBUTTON  = 0x0001
MK_RBUTTON  = 0x0002
MK_SHIFT    = 0x0004
MK_CONTROL  = 0x0008
MK_MBUTTON  = 0x0010
MK_XBUTTON1 = 0x0020
MK_XBUTTON2 = 0x0040

WS_TABSTOP     = 0x00010000
WS_MAXIMIZEBOX = 0x00010000
WS_GROUP       = 0x00020000
WS_MINIMIZEBOX = 0x00020000
WS_SIZEBOX     = 0x00040000
WS_SYSMENU     = 0x00080000
WS_HSCROLL     = 0x00100000
WS_VSCROLL     = 0x00200000
WS_DLGFRAME    = 0x00400000
WS_BORDER      = 0x00800000
WS_CAPTION     = 0x00C00000
WS_MAXIMIZE    = 0x01000000
WS_DISABLED    = 0x08000000
WS_VISIBLE     = 0x10000000
WS_MINIMIZE    = 0x20000000
WS_CHILD       = 0x40000000
WS_POPUP       = 0x80000000
WS_OVERLAPPEDWINDOW = WS_CAPTION | WS_SYSMENU | WS_SIZEBOX | WS_MINIMIZEBOX | WS_MAXIMIZEBOX
WS_POPUPWINDOW = WS_POPUP | WS_BORDER | WS_SYSMENU

CW_USEDEFAULT = -1

SWP_NOSIZE = 1
SWP_NOMOVE = 2
SWP_NOZORDER = 4
SWP_NOREDRAW = 8
SWP_NOACTIVATE = 0x10
SWP_DRAWFRAME = 0x20
SWP_FRAMECHANGED = 0x20
SWP_SHOWWINDOW = 0x40
SWP_HIDEWINDOW = 0x80
SWP_NOCOPYBITS = 0x100
SWP_NOOWNERZORDER = 0x200
SWP_NOREPOSITION = 0x200
SWP_NOSENDCHANGING = 0x400
SWP_RELSIZE = 0x1000
SWP_RELMOVE = 0x2000

SW_HIDE = 0
SW_NORMAL = 1
SW_MAXIMIZE = 3
SW_SHOW = 5
SW_MINIMIZE = 6
SW_RESTORE = 9

RDW_INVALIDATE = 1
RDW_INTERNALPAINT = 2
RDW_ERASE = 4
RDW_VALIDATE = 8
RDW_NOINTERNALPAINT = 16
RDW_NOERASE = 32
RDW_NOCHILDREN = 64
RDW_ALLCHILDREN = 128
RDW_UPDATENOW = 256
RDW_ERASENOW = 512
RDW_FRAME = 1024
RDW_NOFRAME = 2048

WM_NULL = 0
WM_CREATE = 1
WM_DESTROY = 2
WM_MOVE = 3
WM_SIZE = 5
WM_ACTIVATE = 6
WM_SETFOCUS = 7
WM_KILLFOCUS = 8
WM_ENABLE = 10
WM_SETREDRAW = 11
WM_SETTEXT = 12
WM_GETTEXT = 13
WM_GETTEXTLENGTH = 14
WM_PAINT = 15
WM_CLOSE = 16
WM_QUERYENDSESSION = 17
WM_QUIT = 18
WM_QUERYOPEN = 19
WM_ERASEBKGND = 20
WM_SYSCOLORCHANGE = 21
WM_ENDSESSION = 22
WM_SHOWWINDOW = 24
WM_WINDOWPOSCHANGED = 47h
WM_NCPAINT = 85h
WM_NCMOUSEMOVE = 0A0h
WM_NCLBUTTONDOWN = 0A1h
WM_NCLBUTTONUP = 0A2h
WM_NCLBUTTONDBLCLK = 0A3h
WM_NCRBUTTONDOWN = 0A4h
WM_NCRBUTTONUP = 0A5h
WM_NCRBUTTONDBLCLK = 0A6h
WM_NCMBUTTONDOWN = 0A7h
WM_NCMBUTTONUP = 0A8h
WM_NCMBUTTONDBLCLK = 0A9h
WM_NCXBUTTONDOWN = 0ABh
WM_NCXBUTTONUP = 0ACh
WM_NCXBUTTONDBLCLK = 0ADh
WM_KEYDOWN    = 100h
WM_KEYUP      = 101h
WM_CHAR       = 102h
WM_SYSKEYDOWN = 104h
WM_SYSKEYUP   = 105h
WM_COMMAND = 111h
WM_TIMER   = 113h
WM_TERMINATED   = 180h
WM_NEW_CLIP     = 181h
WM_WINDOWCLOSED = 182h
WM_SCROLL       = 183h
WM_MOUSEBUTTON  = 184h
WM_MAINPANELHCHANGED = 185h
WM_NCMOUSEOVER  = 186h
WM_CHARIO       = 187h
WM_PAINTBUTTONS = 188h
WM_MOUSEOVER    = 189h
WM_DISKCHANGED  = 18Ah
WM_USB_INT = 1A0h
WM_MOUSEMOVE = 200h
WM_LBUTTONDOWN = 201h
WM_LBUTTONUP = 202h
WM_LBUTTONDBLCLK = 203h
WM_RBUTTONDOWN = 204h
WM_RBUTTONUP = 205h
WM_RBUTTONDBLCLK = 206h
WM_MBUTTONDOWN = 207h
WM_MBUTTONUP = 208h
WM_MBUTTONDBLCLK = 209h
WM_MOUSEWHEEL = 20Ah
WM_XBUTTONDOWN = 20Bh
WM_XBUTTONUP = 20Ch
WM_XBUTTONDBLCLK = 20Dh
WM_MOUSEHWHEEL = 20Eh
WM_DEVICECHANGE = 219h

